package uwe;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;
import org.eclipse.emf.common.util.EList;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Comment;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.InstanceSpecification;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.Stereotype;

/**
 * 
 * @author Krzysztof Wolf <krzysztof.wolf.1980@googlemail.com>
 * 
 */
public class Helper {
	public static Logger logger = Logger.getRootLogger();
	private static final String CALL_MSG = "\t----- HELPER CALL: ";

	public static void printClass(org.eclipse.uml2.uml.Class clazz) {
		Helper.logger.debug(CALL_MSG + "printClass(" + clazz.getLabel() + ")");
		Helper.logger.debug("  QualifiedName: " + clazz.getQualifiedName());
		Helper.logger.debug("  parents: " + clazz.parents());
		Helper.logger.debug("  allParents: " + clazz.allParents());

		/* Print operations */
		List<Operation> operationList = clazz.getOperations();
		for (Operation operation : operationList) {
			Helper.logger.debug("Operation: " + operation.getLabel());
		}

		printAppliedStereotypes(clazz);
	}

	public static List<org.eclipse.uml2.uml.Class> findNavigationMenusClasses(List<Element> elementList) {
		Helper.logger.debug(CALL_MSG + "findNavigationMenusClasses");
		List<org.eclipse.uml2.uml.Class> classList = new ArrayList<org.eclipse.uml2.uml.Class>();
		List<org.eclipse.uml2.uml.Class> classListAll = Helper.findClasses(elementList, "navigationMenu");
		for (org.eclipse.uml2.uml.Class clazz : classListAll) {
			if (!clazz.isAbstract()) {
				classList.add(clazz);
			}
		}

		return classList;
	}

	public static List<State> findNavigationMenusStates(List<Element> elementList, List<org.eclipse.uml2.uml.Class> classList) {
		Helper.logger.debug(CALL_MSG + "findNavigationMenusState");
		List<State> stateList = new ArrayList<State>();
		for (org.eclipse.uml2.uml.Class clazz : classList) {
			Behavior behavior = Helper.getBehaviorForClass(clazz);
			Helper.logger.debug("Behavior name: " + behavior.getName());
			State state = Helper.findState(elementList, behavior.getName());
			if (state == null){
				logger.error("For the following behavior, no state could be found: " + behavior.getName());
			}
			stateList.add(state);
		}

		return stateList;
	}

	private static Behavior getBehaviorForClass(org.eclipse.uml2.uml.Class clazz) {
		Helper.logger.debug(CALL_MSG + "getBehaviorForClass");
		EList<Behavior> behaviorEList = clazz.getOwnedBehaviors();
		return behaviorEList.get(0);
	}

	public static org.eclipse.uml2.uml.Class findClass(List<Element> elementList, String label) {
		Helper.logger.debug(CALL_MSG + "findClass(" + label + ")");
		for (Element element : elementList) {
			if (getElementType(element).equals("Class")) {
				org.eclipse.uml2.uml.Class clazz = (org.eclipse.uml2.uml.Class) element;
				if (clazz.getLabel().equals(label)) {
					Helper.logger.debug("Found class with label: " + clazz.getLabel());
					return clazz;
				}
			}
		}
		return null;
	}

	private static List<org.eclipse.uml2.uml.Class> findClasses(List<Element> elementList, String stereotypeName) {
		Helper.logger.debug(CALL_MSG + "findClasses(" + stereotypeName + ")");
		List<org.eclipse.uml2.uml.Class> classList = new ArrayList<org.eclipse.uml2.uml.Class>();
		for (Element element : elementList) {
			if (getElementType(element).equals("Class")) {
				org.eclipse.uml2.uml.Class clazz = (org.eclipse.uml2.uml.Class) element;
				if (findAppliedStereotype(element, stereotypeName) != null) {
					Helper.logger.debug("Adding class with label to list: " + clazz.getLabel());
					classList.add(clazz);
				}
			}
		}
		return classList;
	}

	public static State findStateHome(List<Element> elementList) {
		Helper.logger.debug(CALL_MSG + "findStateHome()");
		for (Element element : elementList) {
			if (getElementType(element).equals("State")) {
				State state = (State) element;
				if ((findStereotypeProperty(state, "isHome") + "").equals("true")) {
					return state;
				}
			}
		}
		return null;
	}

	public static Object findStereotypeProperty(Element element, String propertyName) {
		Helper.logger.debug(CALL_MSG + "findStereotypeProperty");
		List<Stereotype> stereotypeList = element.getAppliedStereotypes();
		for (Stereotype stereotype : stereotypeList) {
			List<Property> propertyList = stereotype.getAllAttributes();
			for (Property property : propertyList) {
				if (property != null && property.getName().equals(propertyName)) {
					Object propertyValue = element.getValue(stereotype, propertyName);
					Helper.logger.debug("Found property \"" + propertyName + "\" Value: " + propertyValue);
					// Helper.logger.debug( "Class: " +
					// element.getValue(stereotype,
					// propertyName).getClass().getCanonicalName() );
					return propertyValue;
				}
			}
		}
		return null;
	}

	public static List<InstanceSpecification> findInstanceSpecificationsForObject(List<Element> elementList, String objectName) {
		Helper.logger.debug(CALL_MSG + "findInstanceSpecifications()");
		List<InstanceSpecification> instanceSpecificationList = new ArrayList<InstanceSpecification>();
		for (Element element : elementList) {
			if (getElementType(element).equals("InstanceSpecification")) {
				InstanceSpecification instanceSpecification = (InstanceSpecification) element;
				List<Classifier> classifierList = instanceSpecification.getClassifiers();
				{
					if (classifierList != null && !classifierList.isEmpty()) {
						Classifier classifier = classifierList.get(0);
						if (classifier.getLabel().equals(objectName)) {
							instanceSpecificationList.add(instanceSpecification);
							Helper.logger.debug("Found instanceSpecification for object: " + instanceSpecification.getLabel() + " " + instanceSpecification.getClassifiers());
						}
					}
				}
			}
		}
		return instanceSpecificationList;
	}

	public static List<InstanceSpecification> findInstanceSpecifications(List<Element> elementList, String separator) {
		Helper.logger.debug(CALL_MSG + "findInstanceSpecifications()");
		List<InstanceSpecification> instanceSpecificationList = new ArrayList<InstanceSpecification>();
		for (Element element : elementList) {
			if (getElementType(element).equals("InstanceSpecification")) {
				InstanceSpecification instanceSpecification = (InstanceSpecification) element;
				if (instanceSpecification.getLabel().matches(".*" + separator + ".*")) {
					instanceSpecificationList.add(instanceSpecification);
					Helper.logger.debug("Found instanceSpecification with label: " + instanceSpecification.getLabel());
				}
			}
		}
		return instanceSpecificationList;
	}

	public static List<Dependency> findDependencies(List<Element> elementList, List<String> stereotypeList) {
		Helper.logger.debug(CALL_MSG + "findDependencies()");
		List<Dependency> dependencyList = new ArrayList<Dependency>();
		for (Element element : elementList) {
			if (getElementType(element).equals("Dependency")) {
				if (hasAppliedStereotype(element, stereotypeList)) {
					Dependency dependency = (Dependency) element;
					dependencyList.add(dependency);
					Helper.logger.debug("Found dependency with label and steretype: " + dependency.getLabel());
				}
			}
		}
		return dependencyList;
	}
	
	public static List<Comment> findComments(List<Element> elementList, List<String> stereotypeList) {
		Helper.logger.debug(CALL_MSG + "findComments()");
		List<Comment> commentList = new ArrayList<Comment>();
		for (Element element : elementList) {
			if (getElementType(element).equals("Comment")) {
				if (hasAppliedStereotype(element, stereotypeList)) {
					Comment comment = (Comment) element;
					commentList.add(comment);
					Helper.logger.debug("Found Comment: \"" + comment.getBody() + "\"");
				}
			}
		}
		return commentList;
	}

	public static List<Dependency> filterDependenciesForClient(List<Dependency> elementList, String client) {
		Helper.logger.debug(CALL_MSG + "filterDependenciesForClient(" + client + ")");
		List<Dependency> dependencyList = new ArrayList<Dependency>();
		for (Element element : elementList) {
			if (getElementType(element).equals("Dependency")) {
				Dependency dependency = (Dependency) element;
				if (hasClient(dependency, client)) {
					dependencyList.add(dependency);
					Helper.logger.debug("Found dependency with client: " + client);
				}
			}
		}
		return dependencyList;
	}

	private static State findState(List<Element> elementList, String label) {
		Helper.logger.debug(CALL_MSG + "findState(" + label + ")");
		for (Element element : elementList) {
			if (getElementType(element).equals("State")) {
				State state = (State) element;
				if (state.getLabel().equals(label)) {
					Helper.logger.debug("Found state with label: " + state.getLabel());
					return state;
				}
			}
		}
		return null;
	}

	private static Stereotype findAppliedStereotype(Element element, String stereotypeName) {
		Helper.logger.debug(CALL_MSG + "findAppliedStereotype(" + stereotypeName + ")");
		List<Stereotype> stereotypeList = element.getAppliedStereotypes();
		for (Stereotype stereotype : stereotypeList) {
			if (stereotype.getName().equals(stereotypeName)) {
				Helper.logger.debug("Found stereotype: " + stereotypeName);
				return stereotype;
			}
		}
		return null;
	}

	private static boolean hasAppliedStereotype(Element element, List<String> stereotypeList) {
		Helper.logger.debug(CALL_MSG + "hasAppliedStereotype()");
		List<Stereotype> appliedStereotypeList = element.getAppliedStereotypes();
		for (Stereotype stereotype : appliedStereotypeList) {
			if (stereotypeList.contains(stereotype.getName())) {
				Helper.logger.debug("Has stereotype: " + stereotype.getName());
				return true;
			}
		}
		return false;
	}

	private static boolean hasClient(Dependency dependency, String client) {
		Helper.logger.debug(CALL_MSG + "hasClient(" + client + ")");
		List<NamedElement> clientList = dependency.getClients();
		for (NamedElement namedElement : clientList) {
			if (namedElement.getName().equals(client)) {
				Helper.logger.debug("Has client: " + client);
				return true;
			}
		}
		return false;
	}

	private static void printAppliedStereotypes(Element element) {
		Helper.logger.debug(CALL_MSG + "printAppliedStereotypes");
		List<Stereotype> stereotypeList = element.getAppliedStereotypes();
		for (Stereotype stereotype : stereotypeList) {
			printAppliedStereotype(stereotype, element);
		}
	}

	private static void printAppliedStereotype(Stereotype stereotype, Element element) {
		Helper.logger.debug(CALL_MSG + "printStereotype for \"" + stereotype.getName() + "\"");
		List<Property> propertyList = stereotype.getAllAttributes();
		for (Property property : propertyList) {
			Object propertyValue = element.getValue(stereotype, property.getName());
			Helper.logger.debug("PropertyName: \"" + property.getName() + "\" with value: \"" + propertyValue + "\"");
		}
		Helper.logger.debug("allParents: " + stereotype.allParents() + "\n  parents: " + stereotype.parents());
	}

	private static String getElementType(Element element) {
		return element.eClass().getName();
	}

	public static Logger initLogger() {
		try {
			SimpleLayout layout = new SimpleLayout();
			ConsoleAppender consoleAppender = new ConsoleAppender(layout);
			logger.addAppender(consoleAppender);
			FileAppender fileAppender = new FileAppender(layout, "Transformation.log", false);
			logger.addAppender(fileAppender);
			// ALL | DEBUG | INFO | WARN | ERROR | FATAL | OFF:
			logger.setLevel(Level.INFO);
		} catch (Exception ex) {
			System.err.println(ex);
		}
		return logger;
	}
}